#include <colors.h>
#include <uOLED.h>

// FOREWORD
// A proof-of-concept test sketch for the 4D Systems OLED units.  
// Actual hardware used is the 0.96" units.
// See OLED documentation for pinout, etc.
//
// This sketch assumes that the microSD card on the OLED display has been pre-loaded with
// some "stills".  The OLED is instructed to page through them one at a time in a ring-loop
// in response to a button press.
//
// The original prototype was a "Shot Counter" demo for a gun.  A pseudo-digital readout of numbers (0-10) would 
// count down in response to trigger pulls, and reload back to 10 rounds when a "reload" button was hit.
// 
// Therefore variable names, etc will reflect this original purpose.
//
// Updated Feb 2011 to remove extra code and clean up a bit.  
// donp@aeinnovations.com

/* 
Gotchas
-------
- OLED must be disconnected in order to upload new code to Arduino (since it shares the hardware UART)


Hardware notes
--------------
- Arduino Duemilanove / Uno
- 0.96" OLED (96x64 resolution)
- 12V in max

- Hardware UART to OLED (TX, RX) and with +5.0V, GND for OLED power.
- Analog 0 and 1 to button inputs (Analog 2 is used as digital output LOW for common ground to both switches.)

Software notes
--------------
- Pull trigger = decrement ammo counter
- Hold trigger = autofire until empty (0 rounds remaining) or trigger is released
- Button = reload ammo counter to 10 shots
- OLED is loaded with images which are flipped through via serial commands (this is ammo counter)


OLED Display images
-------------------
- Sample setup:
  // Note: Serial.begin() is not needed
  uoled.begin(12,115200, &Serial);      //reset line connected to pin 12 and a serial speed of 4800, hardware serial 0 (Serial1 or 2 etc for MEGA, etc)
  uoled.DeviceInfo();             // display some hardware/software info of the screen OPTIONAL
  delay(2000);                    // info will be on screen for 2 seconds
  uoled.Cls();                    // clears the screen of everything on it. It uses the background color (at this moment it's at default black)
  uoled.sdInit();                 // Doesn't appear to be needed, sdDisplayImage still works.

- You need to refer to the .txt file made by the graphics composer when the SD card is loaded to find the sector numbers of the images.
  The sector numbers are then sent to the OLED to identify the images (preloaded on the microSD card) to be shown.

- This method works: raw from SD card
      // From ammocounter.txt
      //0x40, 0x49, 0x00, 0x00, 0x60, 0x40, 0x10, 0x00, 0x10, 0x00  // 0x40 (ext cmd), cmd num, x, y, width, height, color mode (16), sector address hi, med, low
      Serial.print(0x40, BYTE);
      Serial.print(0x49, BYTE);
      Serial.print(0x00, BYTE);
      Serial.print(0x00, BYTE);
      Serial.print(0x60, BYTE);
      Serial.print(0x40, BYTE);
      Serial.print(0x10, BYTE);
      Serial.print(0x00, BYTE);
      Serial.print(0x10, BYTE);
      Serial.print(0x00, BYTE);
- Alternately use 
      uoled.sdDisplayImage(0, 0, 96, 64, 1, 0x001000);  // x, y, width, height, boolean twoBytesPP, long sector (sector hi+med+low of image from .txt file)


Connections
-----------
  Analog in 0          Trigger switch (enable internal pullup)
  Analog in 1          Reload switch  (enable internal pullup)
  Analog in 2          common switch LOW/GND (digital pin 16)
  
  RX (digital 0)       TX of OLED
  TX (ditigal 1)       RX of OLED
  
  5V                   to OLED +
  GND                  to OLED -
  
*/



/* CHANGE LOG

2010-09-30    
- OLED displays images
- OLED reset not actually connected to anything (seems OK)
- Version 1.0

2010-10-01
- Removed MP3Trigger, not using sound output.  Simply removed.
- The only change is to alter the sound delays (used for the sound to partially complete so it sounds good) to be much shorter for better trigger response
(after all there is no audio feedback for the user to connect up to the delay).
- Version 1.1

2011-02-08
- Cleaned up code to make more useful as a "hello world"; sent to Brad (renderman).
- Version 1.2



*/



////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Initializations
//
// Things that can be changed are below this line
#define  FIRE_DELAY  50   // Min delay between shots in semi-auto mode (mS)
#define  BURSTDELAY  200  // How long the trigger must be held down before autofire begins (in mS)
#define  AUTOFIRE_DELAY 50  // Min delay between shots in autofire mode (mS)

// Things not normally changed are below this line
#define  VERSION                        1.2
#define  img10                      0x001000    // Sector number (hi+med+low) for this image on SD card (from ammocounter.txt)
#define  img9                      0x001018
#define  img8                      0x001030
#define  img7                      0x001048
#define  img6                      0x001060
#define  img5                      0x001078
#define  img4                      0x001090
#define  img3                      0x0010a8
#define  img2                      0x0010c0
#define  img1                      0x0010d8
#define  img0                      0x0010f0

// Things that should not be changed
#define  switch_Trigger             14     // Analog in 0 (digital 14)
#define  switch_Reload              15     // Analog in 1 (digital 15)

#define  switch_GND                 16     // Analog 2 (digital 16) = common low for switches

#define  OLED_reset                 12     // Digital 12 = reset (NOT CONNECTED)



////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Globals
// 
uOLED uoled;            // Instance of OLED object

int rounds = 10;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Hardware Setup
//
void setup()
{
  // 1 sec for hardware to settle
  delay(1000);
  
  // I/O Settings
  pinMode (OLED_reset, OUTPUT);
  
  pinMode (switch_Trigger, INPUT);
  pinMode (switch_Reload, INPUT);
  pinMode (switch_GND, OUTPUT);

  // Turn on internal pullups for switch inputs
  // (This is done by (digital) writing a HIGH to the pin while it is an input.
  digitalWrite(switch_Trigger, HIGH);        // Enable pullup (done while pin is an input)
  digitalWrite(switch_Reload, HIGH);        // Enable pullup (done while pin is an input)


  // Initial states  
  digitalWrite (switch_GND, LOW);    // Common switch GND is LOW
  //
  digitalWrite (OLED_reset, HIGH);  // NOT ACTUALLY CONNECTED


  // Serial.begin is not necessary (duplicated in uoled.begin)
  // Serial.begin(115200);              // OLED should auto-detect baud rate
  // Reset is unconnected
  uoled.begin(12,115200, &Serial);      //reset line connected to pin 12 and a serial speed of 115200, hardware serial port (the only one on an Uno or Duemilanove)

  uoled.DeviceInfo();             // display some hardware/software info of the screen
  delay(2000);                    // info will be on screen for 2 seconds
  uoled.Cls();                    // clears the screen of everything on it. It uses the background color (at this moment it's at default black)

  //uoled.sdInit();                 // Doesn't appear to be needed, sdDisplayImage still works.
  
  // Finally works - use all 3 sector bytes together for 'long sector' (hi+med+low)
  //uoled.sdDisplayImage(0, 0, 96, 64, 1, 0x001000);  // x, y, width, height, boolean twoBytesPP, long sector
  //uoled.sdDisplayVideo(0, 0, 160, 128, 1, 0x0, 0xB7, 0x1000); x, y, width, height, boolean twoBytesPP, byte fdelay, int frames, long sector)

  updateAmmoDisplay(rounds);    // Display showing will match # of rounds left
}



////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Main Program Loop
//
void loop()
{ 
  while( triggerDown()==false && reloadDown()==false);    // Wait for either button to be pushed
  
  if( triggerDown() == true )
    {
      if( rounds > 0 )
      {
        rounds--;  // Decrement ammo
        updateAmmoDisplay(rounds);
        delay(FIRE_DELAY);
      }

      // Keep checking the trigger status for about 500ms
      // if the trigger is STILL down after that long, then begin autofiring until either empty or trigger released.
      long deadline = millis() + BURSTDELAY;
      while( millis() < deadline )
      {
        if ( triggerDown() == false )
        {
          break;
        }
      }
      // Delay done, is trigger (still) down?
      if( triggerDown() == true )
      {
        while( (triggerDown() == true) && (rounds > 0) )
        {
          // Begin autofire
          rounds--;
          updateAmmoDisplay(rounds);
          delay(AUTOFIRE_DELAY);
        }
      }
      // else we fall out and do not begin (or we are ending) autofire
      while( triggerDown() != false );  // Make sure trigger is back up before going on
    }

  if( reloadDown() == true )
    {
      rounds = 10;  // Reload back to 10
      updateAmmoDisplay(rounds);
      while( reloadDown() != false );
    }


}




////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Functions

// Name:    updateAmmoDisplay(int ammoCount)
// Purpose: Accepts # of rounds and updates the counter (OLED) accordingly
// Returns: False on invalid input, true otherwise
boolean updateAmmoDisplay(int ammoCount)
{
  switch (ammoCount)
  {
    case 0:
      OLED(img0);
      break;
    case 1:
      OLED(img1);
      break;
    case 2:
      OLED(img2);
      break;
    case 3:
      OLED(img3);
      break;
    case 4:
      OLED(img4);
      break;
    case 5:
      OLED(img5);
      break;
    case 6:
      OLED(img6);
      break;
    case 7:
      OLED(img7);
      break;
    case 8:
      OLED(img8);
      break;
    case 9:
      OLED(img9);
      break;
    case 10:
      OLED(img10);
      break;
    default:
      return false;
  }
  return true;
}

// Name:    OLED(long imgSector)
// Returns: true
boolean OLED(long imgSector)
{
  uoled.sdDisplayImage(0, 0, 96, 64, 1, imgSector);
  return true;
}


// Name:     triggerDown()
// Returns:  true or false
boolean triggerDown(void)
{
  if( digitalRead(switch_Trigger) == LOW )    // Trigger is active low
  {
    delay(30);    // 30ms wait (for simple debouncing)
    if( digitalRead(switch_Trigger) == LOW )  // Still low?
      return true;
    else
      return false;
  }
  else
    return false;
}


// Name:     reloadDown()
// Returns:  true or false
boolean reloadDown(void)
{
  if( digitalRead(switch_Reload) == LOW )    // Reload is active low
  {
    delay(30);    // 30ms wait (for simple debouncing)
    if( digitalRead(switch_Reload) == LOW )  // Still low?
      return true;
    else
      return false;
  }
  else
    return false;
}




